Class: SmartSearchHistory
- Inherits:
-
ActiveRecord::Base
- Object
- ActiveRecord::Base
- SmartSearchHistory
- Defined in:
- lib/smart_search_history.rb
Overview
Saves all queries made so the data can be used to build a similarity index based on the queries made by actual users of the website